from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.utils.functional import LazyObject


class PrivateMediaFileSystemStorage(FileSystemStorage):

    """
    Storage that puts files in the private media folder that isn't
    globally available.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('location', settings.PRIVATE_MEDIA_ROOT)
        kwargs.setdefault('base_url', settings.PRIVATE_MEDIA_URL)
        super().__init__(*args, **kwargs)


class PrivateMediaStorage(LazyObject):

    def _setup(self):
        self._wrapped = PrivateMediaFileSystemStorage()


private_media_storage = PrivateMediaStorage()



from django.contrib.admin.widgets import AdminFileWidget
from django.urls import reverse


class PrivateFileWidgetMixin:

    def __init__(self, *args, **kwargs):
        self.url_name = kwargs.pop('url_name')
        super().__init__(*args, **kwargs)

    def get_context(self, name, value, attrs):
        """
        Return value-related substitutions.
        """
        context = super().get_context(name, value, attrs)
        context['url'] = reverse(self.url_name, kwargs={'pk': value.instance.pk}) if value else ''
        return context


class PrivateFileWidget(PrivateFileWidgetMixin, AdminFileWidget):
    template_name = 'admin/widgets/clearable_private_file_input.html'
